﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows;
using System.Windows.Controls;


namespace Prevodnik
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        List<string> languages = new List<string>();
        private string defLang = "cs-CZ";
        private List<Convertor> convertors = new List<Convertor>();
        public MainWindow()
        {
            InitializeComponent();

            //Pridani Convertoru
            convertors.Add(new ConvertorDistance());
            convertors.Add(new ConvertorWeight());
            convertors.Add(new ConvertorData());
            convertors.Add(new ConvertorSquared());
            convertors.Add(new ConvertorCubic());

            //Inicializace
            GetLanguages();
            LangComboBoxInit();
            ConvertComboBoxInit();

        }
        #region Jazyky
        /// <summary>
        /// Do promenne languages doplni jazyky podle souboru ve slozce localization
        /// </summary>
        public void GetLanguages()
        {
            string[] temp;
            var asm = Assembly.GetEntryAssembly();
            string resName = asm.GetName().Name + ".g.resources";
            using (var stream = asm.GetManifestResourceStream(resName))
            using (var reader = new System.Resources.ResourceReader(stream))
            {
                temp = reader.Cast<DictionaryEntry>().Select(entry => (string)entry.Key).ToArray();
            }
            foreach (string s in temp)
            {
                if (s.Contains("localization"))
                {
                    string result = s.Replace("localization/", "").Replace(".baml", ""); // localization/cs-cz.baml -> cs-cz
                    result = result.Split('-')[0] + "-" + result.Split('-')[1].ToUpper(); // cs-cz -> cs-CZ
                    languages.Add(result);
                }
            }
        }
        /// <summary>
        /// Nastavi jazyk
        /// </summary>
        /// <param name="lang">language to set</param>
        private void SetLanguageDictionary(string lang)
        {
            ResourceDictionary dict = new ResourceDictionary();
            string path = "..\\Localization\\{0}.xaml";
            if(languages.Contains(lang))
                dict.Source =  new Uri(String.Format(path, lang), UriKind.Relative);
            else
                dict.Source = new Uri(String.Format(path, defLang), UriKind.Relative);
            Resources.MergedDictionaries.Add(dict);
        }
        private void LangComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var comboBox = (ComboBox)sender;
            SetLanguageDictionary((string)comboBox.SelectedItem);
        }
        #endregion
        #region Inicializace
        //Nastavuje ComboBoxy
        private void ConvertComboBoxInit()
        {
            foreach (Convertor c in convertors)
            {
                if (c.GetType() == "distance")
                {
                    ConvertComboBox.Items.Clear();
                    foreach (string s in c.GetUnits().Keys)
                        ConvertComboBox.Items.Add(s);
                    ConvertComboBox.SelectedItem = c.GetUnits().Keys.First();
                }
            }
        }
        private void LangComboBoxInit()
        {

            LangComboBox.SelectedItem = (languages.Contains(Thread.CurrentThread.CurrentCulture.ToString()))?Thread.CurrentThread.CurrentCulture.ToString():"cs-CZ";
            SetLanguageDictionary((string)LangComboBox.SelectedItem);
            foreach (string s in languages)
                LangComboBox.Items.Add(s);
        }
        #endregion
        private void ConvertComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            Process();
        }
        /// <summary>
        /// Vybere vstupni data
        /// </summary>
        /// <returns>number|unit|target unit</returns>
        private string GetData()
        {
            string data = SourceTextBox.Text.Trim();
            double number = Double.Parse(Regex.Match(data, @"[+-]?[0-9]{1,9}(?:\.[0-9]*)?").Value, CultureInfo.InvariantCulture);
            string unit = data.Replace(number.ToString().Replace(',', '.'), "");
            string target = ConvertComboBox.SelectedItem.ToString();
            return number + "|" + unit + "|" + target;
        }
        /// <summary>
        /// Posle do Convertoru
        /// </summary>
        private void Process()
        {
            string result = "";
            string data = SourceTextBox.Text.Trim();
            if (data == "")
                return;
            string[] dataToProcess = GetData().Split('|');
            double test = Double.Parse(dataToProcess[0]);
            foreach (Convertor c in convertors)
            {
                result = c.Convert(Double.Parse(dataToProcess[0]), dataToProcess[1], dataToProcess[2]);
                if (result != "")
                    break;
            }
            ResultTextBox.Text = result;
        }
        private void SourceTextBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            try
            {
                //Zjistuje zdali na vstupu je jednotka, popr. nastavuje ConvertComboBox
                foreach (Convertor c in convertors)
                    if (c.GetUnits().ContainsKey(GetData().Split('|')[1]))
                    {
                        ConvertComboBoxChange(c.GetType());
                    }
                
            }
            catch (Exception)
            {
            }
        }
        /// <summary>
        /// Zmeni itemy v ConvertComboBoxu na jednotky podle vstupu
        /// </summary>
        /// <param name="type">jednotky</param>
        private void ConvertComboBoxChange(string type)
        {
            
            foreach (Convertor c in convertors)
            {
                if (c.GetType() == type)
                {
                    //Z nejakeho duvodu pri prvnim pruchodu hodi nullreferenceexception
                    try
                    {
                        ConvertComboBoxClear();
                    }
                    catch (Exception)
                    {
                        ConvertComboBoxClear();
                    }
                    //Vlozi moznosti do ConvertComboBoxu dle jednotky na vstupu
                    foreach (string s in c.GetUnits().Keys)
                        ConvertComboBox.Items.Add(s);
                    //Nastavi prvni item jako vybrany(osetreni proti odeslani s prazdnym selectem
                    ConvertComboBox.SelectedItem = c.GetUnits().Keys.First();
                }
            }
        }
        /// <summary>
        /// Vycisti vsechny itemy z ConvertComboBoxu
        /// </summary>
        private void ConvertComboBoxClear()
        {
            for (int i = 0; i < ConvertComboBox.Items.Count; i++)
            {
                ConvertComboBox.Items.RemoveAt(i);
                i--;
            }
        }
        /// <summary>
        /// Vrati vsechny dostupne jednotky s mezerami
        /// </summary>
        /// <returns>All available units with spaces</returns>
        private List<string> GetAvailableUnits()
        {
            List<string> result = new List<string>();
            foreach (Convertor c in convertors)
            {
                foreach (string s in c.GetUnits().Keys)
                    result.Add(s);
                result.Add("---");
            }
            return result;
        }


        private void HelpButton_Click(object sender, RoutedEventArgs e)
        {
            HelpWindow h = new HelpWindow(GetAvailableUnits(), languages, (string)LangComboBox.SelectedItem);
            h.Show();
        }


    }
}
