﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Runtime.InteropServices;

namespace Konvertor
{
    public class Konfigurator
    {
        [DllImport("kernel32.dll")]
        static extern bool FreeConsole();

        /// <summary>
        /// zapnuty/vypnuty mod konzole
        /// </summary>
        public bool ModKonzole { get; private set; }
        /// <summary>
        /// Aktualne zvoleny jazyk
        /// </summary>
        public string Lokalizace { get; private set; }
        /// <summary>
        /// Cesta k aktualne zvolenemu jazyku
        /// </summary>
        public string CestaKLokalizaci { get; private set; }
        /// <summary>
        /// Seznam jazyku v nelokalizovane podobe
        /// </summary>
        public Dictionary<string, string> Jazyky { get; private set; }

        private string _cesta;
        private readonly string _korenovaSlozka;
        private const string PODSLOZKA = "preklady\\";

        /// <summary>
        /// komplexni objekt pro konfiguraci
        /// </summary>
        /// <param name="umisteni">korenovy adresar</param>
        /// <param name="args">argumenty z konzole</param>
        public Konfigurator(string umisteni,string[] args = null)
        {
            _korenovaSlozka = umisteni;
            NactiJazyky();
            ZmenJazyk(JazykSystemu());
            ModKonzole = false;
            if (args != null)
                PredKonfigurace(args);
            string umisteniKonfiguraku = umisteni + (string.IsNullOrEmpty(_cesta) ? "config.ini" : _cesta);
            if (!string.IsNullOrEmpty(umisteniKonfiguraku) && File.Exists(umisteniKonfiguraku))
                NactiKonfiguraciZeSouboru(umisteniKonfiguraku);
            if(args != null)
                NactiArgumenty(args);

            //schova konzoli v pripade ze nejde o mod konzole
            if (!ModKonzole)
                FreeConsole();
        }

        /// <summary>
        /// Nacte konfiguraci ulozenou v souboru (defaultne config.ini)
        /// </summary>
        /// <param name="umisteni">absolutni cesta ke konfigu</param>
        public void NactiKonfiguraciZeSouboru(string umisteni)
        {
            using (StreamReader sr = new StreamReader(umisteni))
            {
                while (!sr.EndOfStream){
                    string line = sr.ReadLine();
                    if (line == null) continue;
                    string[] val = line.Split('=');
                    switch (val[0])
                    {
                        case "localization":
                            if (!string.IsNullOrEmpty(val[1]) && val[1] != "default")
                                ZmenJazyk(val[1]);
                            break;
                        case "console":
                            if (val[1] == "true")
                                ModKonzole = true;
                            break;
                    }
                }
            }
        }

        /// <summary>
        /// Zpracovani argumentu z konzole
        /// </summary>
        /// <param name="args">Argumenty konzole</param>
        public void NactiArgumenty(string[] args)
        {
            for (int i = 0; i < args.Length; i++){
                switch (args[i]){
                    case "-localization":
                        if (args.Length > ++i && args[i][0] != '-')
                            ZmenJazyk(args[i] != "default" ? args[i] : JazykSystemu());
                        else i--;
                        break;
                    //pro mozne prepsani zadosti z konfiguraku
                    case "-gui":
                        ModKonzole = false;
                        break;
                }
            }
        }

        /// <summary>
        /// Zpracovani primarnich konfiguraci (ktere musi byt vyrireny nejdrive)
        /// </summary>
        /// <param name="args">Argumenty konzole</param>
        private void PredKonfigurace(string[] args)
        {
            for (int i = 0; i < args.Length; i++)
            {
                switch (args[i])
                {
                    case "-c":
                        ModKonzole = true;
                        break;
                    //nastavuje cestu ke konfiguraku
                    case "-config":
                        if (args.Length > ++i && args[i][0] != '-')
                            _cesta = args[i];
                        else i--;
                        break;
                }
            }

        }
        
        /// <summary>
        /// Ziska sezna jazyku
        /// </summary>
        private void NactiJazyky()
        {
            Jazyky = new Dictionary<string, string>();
            string[] cesty = Directory.GetFiles(_korenovaSlozka + PODSLOZKA);
            foreach (string jazyk in cesty){
                using (StreamReader sr = new StreamReader(jazyk)){
                    string jmeno = sr.ReadLine();
                    string jmenosouboru = jazyk.Substring(_korenovaSlozka.Length + PODSLOZKA.Length, 2);
                    if (jmeno != null) 
                        Jazyky[jmeno] = jmenosouboru;
                }
            }
        }

        /// <summary>
        /// Prenastavi jazyk
        /// </summary>
        /// <param name="jazyk">Nelokalizovany nazev jazyku</param>
        public void ZmenJazyk(string jazyk)
        {
            if (jazyk == Lokalizace)
                return;
            Lokalizace = jazyk;
            CestaKLokalizaci = _korenovaSlozka + PODSLOZKA + Lokalizace + ".lang";
            if(File.Exists(CestaKLokalizaci))
                return;
            //pro pripadne zacykleni rekurzi
            if(jazyk == "en")
                throw new Exception("neexistujici preklad");
            ZmenJazyk("en");
        }

        /// <summary>
        /// Vrati nelokalizovany jazyk systemu
        /// </summary>
        /// <returns>Nelokalizovany jazyk systemu</returns>
        private string JazykSystemu()
        {
            string jazyk = CultureInfo.CurrentCulture.Name;
            int i = jazyk.IndexOf('-');
            if (i > 0)
                jazyk = jazyk.Substring(0, i);
            return jazyk;
        }
    }
}
